<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Http\Controllers\Traits\MediaUploadingTrait;
use App\Http\Requests\MassDestroyDeploymentOpportunityRequest;
use App\Http\Requests\StoreDeploymentOpportunityRequest;
use App\Http\Requests\UpdateDeploymentOpportunityRequest;
use App\Models\DeploymentOpportunity;
use App\Models\Region;
use App\Models\RequestDatabase;
use App\Models\User;
use Gate;
use Illuminate\Http\Request;
use Spatie\MediaLibrary\MediaCollections\Models\Media;
use Symfony\Component\HttpFoundation\Response;

class DeploymentOpportunitiesController extends Controller
{
    use MediaUploadingTrait;

    public function index()
    {
        abort_if(Gate::denies('deployment_opportunity_access'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $deploymentOpportunities = DeploymentOpportunity::with(['region', 'logistics_administrator', 'location_province_region', 'request'])->get();

        $regions = Region::get();

        $users = User::get();

        $request_databases = RequestDatabase::get();

        return view('admin.deploymentOpportunities.index', compact('deploymentOpportunities', 'regions', 'request_databases', 'users'));
    }

    public function create()
    {
        abort_if(Gate::denies('deployment_opportunity_create'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $regions = Region::pluck('name', 'id')->prepend(trans('global.pleaseSelect'), '');

        $logistics_administrators = User::pluck('name', 'id')->prepend(trans('global.pleaseSelect'), '');

        $location_province_regions = Region::pluck('name', 'id')->prepend(trans('global.pleaseSelect'), '');

        $requests = RequestDatabase::pluck('request_date', 'id')->prepend(trans('global.pleaseSelect'), '');

        return view('admin.deploymentOpportunities.create', compact('location_province_regions', 'logistics_administrators', 'regions', 'requests'));
    }

    public function store(StoreDeploymentOpportunityRequest $request)
    {
        $deploymentOpportunity = DeploymentOpportunity::create($request->all());

        if ($media = $request->input('ck-media', false)) {
            Media::whereIn('id', $media)->update(['model_id' => $deploymentOpportunity->id]);
        }

        return redirect()->route('admin.deployment-opportunities.index');
    }

    public function edit(DeploymentOpportunity $deploymentOpportunity)
    {
        abort_if(Gate::denies('deployment_opportunity_edit'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $regions = Region::pluck('name', 'id')->prepend(trans('global.pleaseSelect'), '');

        $logistics_administrators = User::pluck('name', 'id')->prepend(trans('global.pleaseSelect'), '');

        $location_province_regions = Region::pluck('name', 'id')->prepend(trans('global.pleaseSelect'), '');

        $requests = RequestDatabase::pluck('request_date', 'id')->prepend(trans('global.pleaseSelect'), '');

        $deploymentOpportunity->load('region', 'logistics_administrator', 'location_province_region', 'request');

        return view('admin.deploymentOpportunities.edit', compact('deploymentOpportunity', 'location_province_regions', 'logistics_administrators', 'regions', 'requests'));
    }

    public function update(UpdateDeploymentOpportunityRequest $request, DeploymentOpportunity $deploymentOpportunity)
    {
        $deploymentOpportunity->update($request->all());

        return redirect()->route('admin.deployment-opportunities.index');
    }

    public function show(DeploymentOpportunity $deploymentOpportunity)
    {
        abort_if(Gate::denies('deployment_opportunity_show'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $deploymentOpportunity->load('region', 'logistics_administrator', 'location_province_region', 'request');

        return view('admin.deploymentOpportunities.show', compact('deploymentOpportunity'));
    }

    public function destroy(DeploymentOpportunity $deploymentOpportunity)
    {
        abort_if(Gate::denies('deployment_opportunity_delete'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $deploymentOpportunity->delete();

        return back();
    }

    public function massDestroy(MassDestroyDeploymentOpportunityRequest $request)
    {
        DeploymentOpportunity::whereIn('id', request('ids'))->delete();

        return response(null, Response::HTTP_NO_CONTENT);
    }

    public function storeCKEditorImages(Request $request)
    {
        abort_if(Gate::denies('deployment_opportunity_create') && Gate::denies('deployment_opportunity_edit'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $model         = new DeploymentOpportunity();
        $model->id     = $request->input('crud_id', 0);
        $model->exists = true;
        $media         = $model->addMediaFromRequest('upload')->toMediaCollection('ck-media');

        return response()->json(['id' => $media->id, 'url' => $media->getUrl()], Response::HTTP_CREATED);
    }
}
