<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Http\Controllers\Traits\MediaUploadingTrait;
use App\Http\Requests\MassDestroyRequestDatabaseRequest;
use App\Http\Requests\StoreRequestDatabaseRequest;
use App\Http\Requests\UpdateRequestDatabaseRequest;
use App\Models\Region;
use App\Models\RequestDatabase;
use App\Models\Requester;
use App\Models\RequestType;
use App\Models\RrtRole;
use Gate;
use Illuminate\Http\Request;
use Spatie\MediaLibrary\MediaCollections\Models\Media;
use Symfony\Component\HttpFoundation\Response;

class RequestDatabaseController extends Controller
{
    use MediaUploadingTrait;

    public function index()
    {
        abort_if(Gate::denies('request_database_access'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $requestDatabases = RequestDatabase::with(['requester', 'request', 'technical_requests', 'request_location'])->get();

        $requesters = Requester::get();

        $request_types = RequestType::get();

        $rrt_roles = RrtRole::get();

        $regions = Region::get();

        return view('admin.requestDatabases.index', compact('regions', 'requestDatabases', 'request_types', 'requesters', 'rrt_roles'));
    }

    public function create()
    {
        abort_if(Gate::denies('request_database_create'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $requesters = Requester::pluck('name', 'id')->prepend(trans('global.pleaseSelect'), '');

        $requests = RequestType::pluck('request_type', 'id')->prepend(trans('global.pleaseSelect'), '');

        $technical_requests = RrtRole::pluck('name', 'id');

        $request_locations = Region::pluck('name', 'id')->prepend(trans('global.pleaseSelect'), '');

        return view('admin.requestDatabases.create', compact('request_locations', 'requesters', 'requests', 'technical_requests'));
    }

    public function store(StoreRequestDatabaseRequest $request)
    {
        $requestDatabase = RequestDatabase::create($request->all());
        $requestDatabase->technical_requests()->sync($request->input('technical_requests', []));
        if ($media = $request->input('ck-media', false)) {
            Media::whereIn('id', $media)->update(['model_id' => $requestDatabase->id]);
        }

        return redirect()->route('admin.request-databases.index');
    }

    public function edit(RequestDatabase $requestDatabase)
    {
        abort_if(Gate::denies('request_database_edit'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $requesters = Requester::pluck('name', 'id')->prepend(trans('global.pleaseSelect'), '');

        $requests = RequestType::pluck('request_type', 'id')->prepend(trans('global.pleaseSelect'), '');

        $technical_requests = RrtRole::pluck('name', 'id');

        $request_locations = Region::pluck('name', 'id')->prepend(trans('global.pleaseSelect'), '');

        $requestDatabase->load('requester', 'request', 'technical_requests', 'request_location');

        return view('admin.requestDatabases.edit', compact('requestDatabase', 'request_locations', 'requesters', 'requests', 'technical_requests'));
    }

    public function update(UpdateRequestDatabaseRequest $request, RequestDatabase $requestDatabase)
    {
        $requestDatabase->update($request->all());
        $requestDatabase->technical_requests()->sync($request->input('technical_requests', []));

        return redirect()->route('admin.request-databases.index');
    }

    public function show(RequestDatabase $requestDatabase)
    {
        abort_if(Gate::denies('request_database_show'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $requestDatabase->load('requester', 'request', 'technical_requests', 'request_location');

        return view('admin.requestDatabases.show', compact('requestDatabase'));
    }

    public function destroy(RequestDatabase $requestDatabase)
    {
        abort_if(Gate::denies('request_database_delete'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $requestDatabase->delete();

        return back();
    }

    public function massDestroy(MassDestroyRequestDatabaseRequest $request)
    {
        RequestDatabase::whereIn('id', request('ids'))->delete();

        return response(null, Response::HTTP_NO_CONTENT);
    }

    public function storeCKEditorImages(Request $request)
    {
        abort_if(Gate::denies('request_database_create') && Gate::denies('request_database_edit'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $model         = new RequestDatabase();
        $model->id     = $request->input('crud_id', 0);
        $model->exists = true;
        $media         = $model->addMediaFromRequest('upload')->toMediaCollection('ck-media');

        return response()->json(['id' => $media->id, 'url' => $media->getUrl()], Response::HTTP_CREATED);
    }
}
