<?php

namespace App\Models;

use \DateTimeInterface;
use Carbon\Carbon;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use Spatie\MediaLibrary\HasMedia;
use Spatie\MediaLibrary\InteractsWithMedia;
use Spatie\MediaLibrary\MediaCollections\Models\Media;

class DeploymentOpportunity extends Model implements HasMedia
{
    use SoftDeletes;
    use InteractsWithMedia;
    use HasFactory;

    public $table = 'deployment_opportunities';

    protected $dates = [
        'expected_deployment_start_date',
        'actual_deployment_start_date',
        'expected_deployment_end_date',
        'actual_deployment_end_date',
        'created_at',
        'updated_at',
        'deleted_at',
    ];

    protected $fillable = [
        'name',
        'region_id',
        'posting_number',
        'logistics_administrator_id',
        'event_name',
        'deployer_position_title',
        'expected_deployment_start_date',
        'actual_deployment_start_date',
        'expected_deployment_end_date',
        'actual_deployment_end_date',
        'number_of_deployers',
        'location_city',
        'location_province_region_id',
        'language_required',
        'request_id',
        'created_at',
        'updated_at',
        'deleted_at',
    ];

    public function registerMediaConversions(Media $media = null): void
    {
        $this->addMediaConversion('thumb')->fit('crop', 50, 50);
        $this->addMediaConversion('preview')->fit('crop', 120, 120);
    }

    public function region()
    {
        return $this->belongsTo(Region::class, 'region_id');
    }

    public function logistics_administrator()
    {
        return $this->belongsTo(User::class, 'logistics_administrator_id');
    }

    public function getExpectedDeploymentStartDateAttribute($value)
    {
        return $value ? Carbon::parse($value)->format(config('panel.date_format')) : null;
    }

    public function setExpectedDeploymentStartDateAttribute($value)
    {
        $this->attributes['expected_deployment_start_date'] = $value ? Carbon::createFromFormat(config('panel.date_format'), $value)->format('Y-m-d') : null;
    }

    public function getActualDeploymentStartDateAttribute($value)
    {
        return $value ? Carbon::parse($value)->format(config('panel.date_format')) : null;
    }

    public function setActualDeploymentStartDateAttribute($value)
    {
        $this->attributes['actual_deployment_start_date'] = $value ? Carbon::createFromFormat(config('panel.date_format'), $value)->format('Y-m-d') : null;
    }

    public function getExpectedDeploymentEndDateAttribute($value)
    {
        return $value ? Carbon::parse($value)->format(config('panel.date_format')) : null;
    }

    public function setExpectedDeploymentEndDateAttribute($value)
    {
        $this->attributes['expected_deployment_end_date'] = $value ? Carbon::createFromFormat(config('panel.date_format'), $value)->format('Y-m-d') : null;
    }

    public function getActualDeploymentEndDateAttribute($value)
    {
        return $value ? Carbon::parse($value)->format(config('panel.date_format')) : null;
    }

    public function setActualDeploymentEndDateAttribute($value)
    {
        $this->attributes['actual_deployment_end_date'] = $value ? Carbon::createFromFormat(config('panel.date_format'), $value)->format('Y-m-d') : null;
    }

    public function location_province_region()
    {
        return $this->belongsTo(Region::class, 'location_province_region_id');
    }

    public function request()
    {
        return $this->belongsTo(RequestDatabase::class, 'request_id');
    }

    protected function serializeDate(DateTimeInterface $date)
    {
        return $date->format('Y-m-d H:i:s');
    }
}
